#!/bin/sh
###########################################################################
#                                                                         #
#                                                                         #
#              Copyright (c) 2008 Iskratel                                #
#                                                                         #
#                                                                         #
#   Name        : ncfm_func.sh                                            #
#                                                                         #
#   Description : Set operating system configuration files                #
#                 (for linux products)                                    #
#                                                                         #
#   Position    : $RTE_DIR/system/ncfm/                                   #
#                                                                         #
#   Code        : QQQL                                                    #
#                                                                         #
#   Date        : may 2009                                                #
#                                                                         #
#   Author      : Andrej Oblak, ITRDD1                                    #
#                                                                         #
#   Translation : * None                                                  #
#                                                                         #
###########################################################################

#All global variables are written with UPPER CAPITALS and are formatet like:
# letter an underscore - example: G_RV - gloval return value
# meaning of first leters:
# G_   -   global
# F_   -   files
# D_   -   direcroty
# DG_  -   debug
# X_   -   executable
# E_   -   error
# C_   -   linux configuration file
# STR_ -   parsing strings


##################################
# Load init script configuration #
##################################
if [ -f "$F_CONF_FILE" ]
then
  . $F_CONF_FILE
else
  echo "No configuration file: $F_CONF_FILE"
  echo "No configuration file: $F_CONF_FILE" >> $F_DEBUGFILE
  exit $E_NO_CONF_FILE
fi

#######################################
# Load product specific configuration #
#######################################
if [ -f "$F_PROD_CONF_FILE" ]
then
  . $F_PROD_CONF_FILE
else
  echo "No product specific configuration file: $F_PROD_CONF_FILE" >> $F_DEBUGFILE
  #we can try to do something without product specific configuration file
fi

#####################################################################################
#*********  G L O B A L   F U N C T I O N S   I M P L E M E N T A T I O N  *********#
#####################################################################################
#every function prefix is: ncfm_, so by this prefix we know we know this is function

#set debugging on specified device
ncfm_set_debug()
{
  F_DEBUGFILE=$1
}

#reset debugging on initialized value
ncfm_reset_debug()
{
  F_DEBUGFILE=$F_DEBUGFILE_BACKUP
}

#print string on F_DEBUGFILE
ncfm_printf()
{
  echo $1 >> $F_DEBUGFILE
}

#print string on F_ERR_FILE
ncfm_printErr()
{
  echo $1 >> $F_DEBUGFILE
  echo $1 >> $F_ERR_FILE
}

#check if value is numeric
ncfm_check_if_numeric()
{
  #check if we have one argument
  if [ $# -lt 1 ]
  then
    ncfm_printErr "ncfm_check_if_numeric: One input argument is needed!"
    return $E_BAD_ARGS
  fi
  
  case $1 in
    *[^0-9]*)
    ncfm_printErr "ncfm_check_if_numeric: Input argument is not integer!"
    return $E_ERROR;
    ;;
  esac
  
  return $OK;
}

#print the usage of ncfm.sh
ncfm_usage()
{
  local objects=`echo $G_ENABLED_OBJECTS | $X_SED -e 's: : | :g'`
  local actions=`echo $G_AVAILABLE_ACTIONS_LONG | $X_SED -e 's: : | :g'`
  
  local object=0
  if [ $# -lt 1 ]
  then
    set -- $objects
  fi
  
  echo "Usage: `$X_BASENAME $0` action OBJECT [-N] [-O]
  DESCRIPTION
    Note that none OBJECT have all actions!
    
    action := { $actions }
    
    OBJECT action will be made over OBJECT
    OBJECT := { $objects }
    
  OBJECT DESCRIPTION:"
  
  while [ ! -z "$1" ]
  do
    object=$1
    shift
  
  case "$object" in
  
  hosts)
  echo "
    hosts:
      It only needs two values. First is IP address and the second is host name.
      Values example: 192.168.0.12,ES
      Available actions: insert, update, delete
      Usage example: ncfm.sh update hosts -N 192.168.0.12,ES"
      ;;
      
  host-name)
  echo "
    host-name:
      It only needs one value. That value is actually a one ward long string - host name.
      Available actions: set, update, delete
      Usage example: ncfm.sh update host-name -N HostName"
      ;;
 
 interface)
  echo "
    interface:
      Number of parameters depends on used actions. Arguments on IP actions are:
              - firs:IP addres,
              - second: network mask in CIDR or dot notation (example: /8 or 255.0.0.0),
              - third: VLAN. If you don't need VLAN you can put 0 for VLAN,
              - fourth: interface name without VLAN (example: eth0).
      Argumens on other actions are:
              - firs: VLAN. If you don't need VLAN you can put 0 for VLAN,
              - second: interface name without VLAN (example: eth0).
      Available actions: insert, update, delete, attach-ip, update-ip, delete-ip, up, down
      Usage example 1: ncfm.sh attach-ip interface -N 192.168.0.50,/24,0,eth0
      Usage example 2: ncfm.sh attach-ip interface -N 192.168.2.51,255.255.0.0,2005,eth1
      Usage example 3: ncfm.sh up interface -N 34,eth0
      Usage example 4: ncfm.sh insert interface -N 4020,eth1"
      ;;
 
 route)
  echo "
    route:
      It needs three parameters: 
              - firs: destination (IP or NETWORK or default),
              - second: network mask in CIDR or dot notation (example: /8 or 255.0.0.0).
              - third: gateway ip address,
      Available actions: insert, update, delete
      Usage example 1: ncfm.sh insert route -N default,0,192.169.1.1
      Usage example 2: ncfm.sh insert route -N 0.0.0.0,0,192.169.1.1
      Usage example 3: ncfm.sh insert route -N 80.0.0.0,255.0.0.0,192.169.1.1"
      ;;
 
 ipmi-board-pos)
  echo "
    ipmi-board-pos:
      It doesn't neeed any value.
      Available actions: get
      Usage example: ncfm.sh get ipmi-board-pos"
      ;;
  
  snmp-trap-tos)
  echo "
    snmp-trap-tos:
      It only needs two values. First attribute tells if we have DSCP value or TOS (TOS field from IP head in decimal nottion) value.
      Available actions: update
      Usage example 1: ncfm.sh update snmp-trap-tos -N tos,72
      Usage example 2: ncfm.sh update snmp-trap-tos -N dscp,18"
      ;;
  
  time-zone)
  echo "
    time-zone:
      It only needs two values. First attribute (zone or offset) specifies the format of the second attribute.
      Available actions: update
      Usage example 1: ncfm.sh update time-zone -N zone,CET
      Usage example 2: ncfm.sh update time-zone -N offset,-18000"    
      ;;
  
  esac #end case
  
  done #end while
  
  echo "      
  ACTION OPTIONS (short way to set action):
        -S
            set
        -C
            create
        -I
            insert
        -U
            update
        -D
            delte
        -G
            get
        -Aip
            attach-ip
        -Uip
            update-ip
        -Dip
            delte-ip
        -up
            up
        -do
            down
        
  OPTIONS
        -O value1,value2,...
            specifies old values
        -N value1,value2,...
            specifies new values"
}

#we set global variable G_RV and exit on error
ncfm_exit_on_error()
{
  G_RV=$? #variable $? has now changed to 0, because we have read it ones
  #this must be first line of this function!!!
  
  local printUsage=0
  #ncfm_printf "G_RV = $G_RV, OK = $OK, \$? = $?"
  if [ "$G_RV" -ne "$OK" ]
  then
    [ "$G_PRINT_USAGE_ON_EXIT" == "1" ] && [ "$G_RV" -eq "$E_BAD_ARGS" ] && printUsage=1
    
    #error occured, so we need to do a rollback if rollback is needed!
    ncfm_rollback $G_BACKUP_FILES
    ncfm_destroy
    
    if [ ! -z "$1" ]
    then
      ncfm_printErr "$1. Error status:$G_RV"
      [ "$printUsage" == "1" ] && ncfm_usage
    else
      ncfm_printErr "Error status:$G_RV"
      [ "$printUsage" == "1" ] && ncfm_usage
    fi
    exit $G_RV
  fi
}
#sed called from c system call from uprpd generated strange error:
#with error 4 /bin/sed: Couldn't close stdout: Bad file descriptor
ncfm_sed_exit_on_error()
{
  G_RV=$? #variable $? has now changed to 0, because we have read it ones
  #this must be first line of this function!!!
  
  local printUsage=0
  #ncfm_printf "G_RV = $G_RV, OK = $OK, \$? = $?"
  if [ "$G_RV" -ne "$OK" ]
  then
  if [ "$G_RV" -ne "4" ]
  then
    [ "$G_PRINT_USAGE_ON_EXIT" == "1" ] && [ "$G_RV" -eq "$E_BAD_ARGS" ] && printUsage=1
    
    #error occured, so we need to do a rollback if rollback is needed!
    ncfm_rollback $G_BACKUP_FILES
    ncfm_destroy
    
    if [ ! -z "$1" ]
    then
      ncfm_printErr "$1. Error status:$G_RV"
      [ "$printUsage" == "1" ] && ncfm_usage
    else
      ncfm_printErr "Error status:$G_RV"
      [ "$printUsage" == "1" ] && ncfm_usage
    fi
    exit $G_RV
  fi
  fi
}

#check used programs, if one is missing error is returned
ncfm_check_fs()
{
  local cmd=0
  
  while [ ! -z "$1" ]
  do
    cmd=$1
    shift
#    cmd=`which $cmd`
#    ncfm_exit_on_error #which not found
    
    #we actually don't need to check command which, cause first check will fail if which does not exists, but we did checked which
    #we check if command file exists
    if [ ! -s "$cmd" ]
    then
      ncfm_printErr "Critial executable file is missing: $cmd"
      return $E_FILE_SYSTEM_ERROR
    fi
    #now we check that file is executable
    if [ ! -x "$cmd" ]
    then
      ncfm_printErr "Critial executable file is not executable: $cmd"
      return $E_FILE_SYSTEM_ERROR
    fi
  done #end while
  return $OK
}

#check configuration files, if one is missing or is not writable error is returned
ncfm_check_confs()
{
  local conf=0
  
  while [ ! -z "$1" ]
  do
    conf=$1
    shift
    
    #we check if configuration file exists (file is not zero size)
    if [ ! -s "$conf" ]
    then
      ncfm_printErr "Critial configuration file is missing: $conf"
      nfm_create_conf_file $conf $C_auto_create_if_needed
      ncfm_exit_on_error "It was not possible to create critical conf file: $fileToCreate"
    fi
    #now we check that file is writable
    if [ ! -w "$conf" ]
    then
      ncfm_printErr "Critial configuration file don't have writable permitions: $conf"
      return $E_FILE_SYSTEM_ERROR
    fi
    
  done #end while
  return $OK
}

#we can automaticaly create missing file if we are allowed to do this
#TODO: create subdirectories if needed
nfm_create_conf_file()
{
  local conf=0
  fileToCreate=$1
  
  shift
  
  while [ ! -z "$1" ]
  do
    conf=$1
    shift
    
    if [ "$fileToCreate" == "$conf" ] #we are allow to create this file
    then
      > $fileToCreate
      if [ ! -e "$fileToCreate" ] #check if file exists
      then
        #ncfm_printErr "It was not possible to create critical conf file: $fileToCreate"
        return $E_FILE_SYSTEM_ERROR
      fi
    fi
  done #end while
  return $OK
  
}
ncfm_selective_check_fs()
{
  local files=""
  
  #we dont check for non critical executable files:
  #X_BASENAME - ncfm_conf.sh (debug purpose), ncfm_usage
  
  #left in global FS check:
  #X_WHICH    - always: for path to the executable files
  #X_SED      - ncfm_read_params, ncfm_usage, ncfm_rollback_delete_from_stack
  #X_RM       - ncfm_destroy, ncfm_add_snmp_trap_tos, ncfm_update_time_zone
  #X_MKDIR    - ncfm_init
  #X_ECHO     - needed all the time
  
  #selectiove FS check:
  #X_GREP     - ncfm_restart_service
  #X_WC       - ncfm_restart_service
  #X_PS       - ncfm_restart_service
  #X_CAT      - ncfm_rollback (for every configuration fil insied ncfm_exit_on_error), ncfm_add_snmp_trap_tos
  #X_CP       - ncfm_rollback_add_to_stack
  #X_VCONFIG  - ncfm_interface_set_state
  #X_IP       - ncfm_interface_ip, ncfm_interface_set_state, ncfm_route_ip
  
  case "$G_NCFM_OBJECT" in
  
    hosts)
      files=`echo $X_CAT $X_CP`
      ;;
  
    host-name)
      files=`echo $X_CAT $X_CP $X_HOSTNAME`
      ;;
  
    interface)
      files=`echo $X_VCONFIG $X_IP`
      ;;
  
    route)
      files=`echo $X_IP`
      ;;
  
    ipmi-board-pos)
      files=`echo $X_IPMICLI`
      ;;
  
    snmp-trap-tos)
      files=`echo $X_NIESUBAG $X_GREP $X_CAT $X_CP $X_WC $X_PS`
      ;;
  
    time-zone)
      files=`echo $X_CAT $X_CP $X_LN`
      ;;
  
    *)#default
      ncfm_printErr "Wrong OBJECT"
      ncfm_usage
      exit $E_BAD_ARGS
      ;;
  
  esac #end case
  
  ncfm_check_fs $files
  ncfm_exit_on_error "ncfm_selective_check_fs: ncfm_check_fs failed"
  
}
ncfm_selective_check_confs()
{
  local files=""
  
  #left in global conf check:
  #X_WHICH    - 
  
  #selectiove FS check:
  #X_GREP     - ncfm_restart_service
  
  case "$G_NCFM_OBJECT" in
  
    hosts)
      files=`echo $C_HOSTS`
      ;;
  
    host-name)
      files=`echo $C_HOSTNAME`
      ;;
  
    interface)
      #files=`echo $`
      ;;
  
    route)
      #files=`echo $`
      ;;
  
    ipmi-board-pos)
      #files=`echo $`
      ;;
  
    snmp-trap-tos)
      files=`echo $C_NIESUBAGENT`
      ;;
  
    time-zone)
      #files=`echo $`
      ;;
  
    *)#default
      ncfm_printErr "Wrong OBJECT"
      ncfm_usage
      exit $E_BAD_ARGS
      ;;
  
  esac #end case
  
  ncfm_check_confs $files
  ncfm_exit_on_error "ncfm_selective_check_confs: ncfm_check_fs failed"
}

ncfm_check_dir()
{
  local dir=0
  
  while [ ! -z "$1" ]
  do
    dir=$1
    shift
    
    #we check if DIR exists and is a directory
    if [ ! -d "$dir" ]
    then
      ncfm_printErr "NCFM direcroty ($dir) was not created"
      return $E_INTERNAL
    fi
    
    #now we check that DIR is writable
    if [ ! -w "$dir" ]
    then
      ncfm_printErr "NCFM directory ($dir) is not writable!"
      return $E_INTERNAL
    fi
  done #end while
  return $OK
}

#check if OBJECT is enabled
ncfm_check_object()
{
  local allObj=0
#  local left=0
  
  #first argument is OBJECT we have got and are looking if it is enabled
  local object=$1
  shift
  
  while [ ! -z "$1" ]
  do
    allObj=$1
    shift
 #   left=$@
    
    #we compare first argument with all others
    [ "$object" == "$allObj" ] && return $OK
    
  done #end while
  
  return $E_BAD_ARGS
}

#check if ACTION is available
ncfm_check_action()
{
  local allActions=0
  
  #first argument is ACTION we have got and are looking if it is available
  local action=$1
  shift
  
  while [ ! -z "$1" ]
  do
    allActions=$1
    shift
    
    #we compare first argument with all others
    [ "$action" == "$allActions" ] && return $OK
    
  done #end while
  
  return $E_BAD_ARGS
}

ncfm_init()
{
  $X_MKDIR -p $G_TMP_DIR >> $F_DEBUGFILE 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_init: NCFM tmp dir cannot be created"
  
  ncfm_check_dir $G_TMP_DIR
  ncfm_exit_on_error "ncfm_init: NCFM ncfm_check_dir failed"
  
  return $OK
}

# this function must not use function ncfm_exit_on_error or any function that is calling ncfm_exit_on_error!!!
# cause this would make endless loop (ncfm_exit_on_error is calling ncfm_rollback)
ncfm_destroy()
{
  $X_RM -rf $G_TMP_DIR >> $F_DEBUGFILE 2>>$F_ERR_FILE
  
  return $OK
}

######################
# Rollback Functions #
######################

# Perform a rollback of configuration files!
# You can specifie as many configuration files as you want.
# List of configuration files must be EVEN number! Fist you specifie a file that is ok and then a file that needs to be replaced!
# this function must not use function ncfm_exit_on_error or any function that is calling ncfm_exit_on_error!!!
# cause this would make endless loop (ncfm_exit_on_error is calling ncfm_rollback)
ncfm_rollback()
{
  local criticalFile=0
  local OKFile=0
  
  while [ ! -z "$1" ]
  do
    OKFile=$1
    shift
    
    #we check if temp configuration file exists
    if [ ! -s "$OKFile" ]
    then
      ncfm_printErr "Internal error while doing a rollback. Tmp file ($OKFile) does not exists"
      return $E_INTERNAL
    fi
    
    if [ -z "$1" ]
    then
      ncfm_printErr "Internal error while doing a rollback. Rollback is not used properly! You should specifie one more conf file!"
      return $E_INTERNAL
    fi
    
    criticalFile=$1
    shift
    
    #now we change a good file with bad one: content change
    $X_CAT $OKFile > $criticalFile 2>>$F_ERR_FILE
    
  done #end while
  return $OK
}

#this function adds configuration file to the stack and make a copy of original file
#first argument is copy file and the second is original file
#function excepts only two inpurt arguments
ncfm_rollback_add_to_stack()
{
  #check if we have exactly two input arguments
  if [ ! $# -eq 2 ]
  then
    ncfm_printErr "Wrong usage of function: ncfm_rollback_add_to_stack"
    return $E_INTERNAL
  fi
  
  if [ -z "$1" ]
  then
    ncfm_printErr "Tmp file ($1) already exists! Maybe you are doing something wrong. NCFM will continue anyway"
  fi
    
  #make a file copy
  $X_CP $2 $1 >> $F_DEBUGFILE 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_rollback_add_to_stack: cp failed"
  
  #now we add file to the stack
  G_BACKUP_FILES=${G_BACKUP_FILES}"$1 $2 "
  
  return $OK
}

#this function adds configuration file to the stack and make a kopi of original file
ncfm_rollback_delete_from_stack()
{
  #check if we have exactly two input arguments
  if [ ! $# -eq 2 ]
  then
    ncfm_printErr "Wrong usage of function: ncfm_rollback_delete_from_stack"
    return $E_INTERNAL
  fi
  
  #let's delete first argument from stack
  #G_BACKUP_FILES=${G_BACKUP_FILES/$1 }
  G_BACKUP_FILES=`echo $G_BACKUP_FILES | $X_SED -e "s:$1::"`
  
  #now we delete second argument from stack
  #G_BACKUP_FILES=${G_BACKUP_FILES/$2 }
  G_BACKUP_FILES=`echo $G_BACKUP_FILES | $X_SED -e "s:$2::"`
  
  if [ ! -z "$G_BACKUP_FILES" ]
  then
    G_BACKUP_FILES=${G_BACKUP_FILES}" "
  fi
  
  return $OK
}

############################
# Rollback Functions - END #
############################


#########################
# Read input parameters #
#########################
ncfm_read_params()
{
  local newSet=0
  local oldSet=0
  
  #$1 should be avaliable action
  G_NCFM_ACTION=$1
  ncfm_check_action $G_NCFM_ACTION $G_AVAILABLE_ACTIONS
  ncfm_exit_on_error "Wrong input parameters: wrong ACTION!"
  case "$1" in
    -S | -I | set | create)
      G_NCFM_ACTION="insert"
      ;;
    -U)
      G_NCFM_ACTION="update"
      ;;
    -D)
      G_NCFM_ACTION="delete"
      ;;
    -G)
      G_NCFM_ACTION="get"
      ;;
    -Aip)
        G_NCFM_ACTION="attach-ip"
      ;;
    -Uip)
        G_NCFM_ACTION="update-ip"
      ;;
    -Dip)
        G_NCFM_ACTION="delte-ip"
      ;;
    -up)
        G_NCFM_ACTION="up"
      ;;
    -do)
        G_NCFM_ACTION="down"
      ;;
  esac #end case
  
  G_NCFM_OBJECT=$2
  
  ncfm_check_object $G_NCFM_OBJECT $G_ENABLED_OBJECTS
  ncfm_exit_on_error "Wrong input parameters: wrong OBJECT!"
  
  [ "$G_NCFM_ACTION" == "get" -o  "$G_NCFM_ACTION" == "help" ] && return $OK
  
  if [ "$3" != "-N" ]&&[ "$3" != "-O" ]
  then
    ncfm_printErr "Wrong input parameters: options needed!"
    return $E_BAD_ARGS
  fi
  
  if [ "$3" == "-N" ]
  then
    G_NCFM_NEW_OPTIONS=$4
    newSet=1
  else
    if [ "$3" == "-O" ]
    then
      G_NCFM_OLD_OPTIONS=$4
      oldSet=1
    fi
  fi
  
  if [ "$5" == "-N" ]
  then
    G_NCFM_NEW_OPTIONS=$6
    newSet=1
  else
    if [ "$5" == "-O" ]
    then
      G_NCFM_OLD_OPTIONS=$6
      oldSet=1
    fi
  fi
  
  #change delimiter
  G_NCFM_NEW_OPTIONS=`echo $G_NCFM_NEW_OPTIONS | $X_SED -e 's/\,/ /g'`
  G_NCFM_OLD_OPTIONS=`echo $G_NCFM_OLD_OPTIONS | $X_SED -e 's/\,/ /g'`
  
  #echo G_NCFM_ACTION = $G_NCFM_ACTION, G_NCFM_OBJECT = $G_NCFM_OBJECT, G_NCFM_NEW_OPTIONS = $G_NCFM_NEW_OPTIONS, G_NCFM_OLD_OPTIONS =$G_NCFM_OLD_OPTIONS
  
  #we check if values for new parameters has been set they always must be set
  if [ -z "$G_NCFM_NEW_OPTIONS" ]
  then
    ncfm_printErr "Error while parsing input parameters: New values has not been set"
    return $E_BAD_ARGS
  fi
  
  #we check if values for old parameters has been set
  if [ "$oldSet" == "1" ]&&[ -z "$G_NCFM_OLD_OPTIONS" ]
  then
    ncfm_printErr "Error while parsing input parameters: Old values has not been set"
    return $E_BAD_ARGS
  fi
  
  return $OK
}

#############################################
# Selective chech number of input arguments #
#############################################
ncfm_selective_arg_chk()
{
  local argNum=$G_ARGS_NUM

  case "$G_NCFM_ACTION" in

    get)
      ;;
    
    help)
      ncfm_usage $G_NCFM_OBJECT
      exit $OK
      ;;
    
    *)#default
      : $((argNum = $argNum + 2)) #increment
      ;;

  esac #end case
  
  if [ "$#" -lt "$argNum" ]
  then
    exit $E_BAD_ARGS
  fi
}

########################
# NCFM restart service #
########################
# input: 1. arg: file name of service or string to search to find if service is running
#        2. arg: executable file to start or most likely restart service
#        3. arg: one argument for executable file of service most likey this value will be restart or less likely start
ncfm_restart_service()
{
  
  #check if we have exactly three input arguments
  if [ ! $# -eq 3 ]
  then
    ncfm_printErr "Wrong usage of function: ncfm_restart_service"
    return $E_INTERNAL
  fi
  
  local fileName=$1
  local service=$2
  local serviceAction=$3
  
  local tmp=0
  local inc=0
  local error=0
  
  while [ "$inc" -lt "$G_RESTART_RETIES" ]
  do
    : $((inc = $inc + 1)) #increment
    
    #execute service with action...most likely it will be restart
    $service $serviceAction
    
    tmp=`$X_PS ax | $X_GREP $fileName | $X_GREP -v grep | $X_WC -l`
    
    #service is not running, something must be wrong
    if [ $tmp -lt 1 ]
    then
      error=1
    fi
    
    [ $error -eq 0 ] && break
  done #end while
  
  if [ $error -eq 1 ]
  then
    ncfm_printErr "Service $fileName is not running"
    return $E_SERVICE_ERROR
  fi
    
  return $OK
}


###########################
# HOSTS FILE (/etc/hosts) #
###########################
ncfm_add_hosts()
{
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have atleast two arguments and no more than 3
  if [ $# -lt 2 ] || [ $# -gt 4 ]
  then
    return $E_BAD_ARGS
  fi
  
  $X_ECHO $G_NCFM_NEW_OPTIONS >> $C_HOSTS 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_add_hosts: echo failed"
  return $OK
}

ncfm_delete_hosts()
{
  local hostName
  set -- $G_NCFM_NEW_OPTIONS
    
  if [ -n "$2" ]
  then
    hostName=$2
  else
    if [ -n "$1" ]
    then
      hostName=$1
    else
      #error - no parameter specified
      return $E_BAD_ARGS
    fi
  fi
  
  $X_SED -i "/\<$hostName\>/ d" $C_HOSTS 2>>$F_ERR_FILE
  ncfm_sed_exit_on_error "ncfm_delete_hosts: sed failed"
  return $OK
}

ncfm_update_hosts()
{
  ncfm_delete_hosts
  ncfm_exit_on_error "ncfm_delete_hosts failed"
  ncfm_add_hosts
  ncfm_exit_on_error "ncfm_add_hosts failed"
  return $OK
}

ncfm_hosts()
{
  ncfm_rollback_add_to_stack $C_HOSTS_TMP $C_HOSTS
  ncfm_exit_on_error "ncfm_hosts: ncfm_rollback_add_to_stack failed"
  
  case "$G_NCFM_ACTION" in

    insert)
      ncfm_add_hosts
      ncfm_exit_on_error "ncfm_add_hosts failed"
      ;;
  
    update)
      ncfm_update_hosts
      ncfm_exit_on_error "ncfm_update_hosts failed"
      ;;
     
    delete)
      ncfm_delete_hosts
      ncfm_exit_on_error "ncfm_delete_hosts failed"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  ncfm_rollback_delete_from_stack $C_HOSTS_TMP $C_HOSTS
  ncfm_exit_on_error "ncfm_hosts: ncfm_rollback_delete_from_stack failed"
  
  return $OK
}
####################
# HOSTS FILE - END #
####################


#############
# HOST NAME #
#############
#how to find host name files:
#find /etc -type f -exec grep "oldHostName" {} \; -print

ncfm_add_host_name()
{
  local hostName
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have one argument
  if [ $# -lt 1 ]
  then
    return $E_BAD_ARGS
  fi
    
  if [ -n "$1" ]
  then
    hostName=$1
  else
    #error - no parameter specified
    return $E_BAD_ARGS
  fi
  
  $X_ECHO "$G_NCFM_NEW_OPTIONS" > $C_HOSTNAME 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_add_host_name: echo failed"
  
  $X_HOSTNAME "$G_NCFM_NEW_OPTIONS" >> $F_DEBUGFILE 2>>$F_ERR_FILE
  ncfm_exit_on_error "Setting hostname failed"
  
  return $OK
}

ncfm_delete_host_name()
{
  $X_ECHO "" > $C_HOSTNAME 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_delete_host_name: Echo failed"
  
  $X_HOSTNAME "" >> $F_DEBUGFILE 2>>$F_ERR_FILE
  ncfm_exit_on_error "Setting empty host name failed"
  
  return $OK
}

ncfm_update_host_name()
{
  ncfm_delete_host_name
  ncfm_exit_on_error "ncfm_delete_host_name failed"
  ncfm_add_host_name
  ncfm_exit_on_error "ncfm_add_host_name failed"
  return $OK
}

ncfm_host_name()
{
  ncfm_rollback_add_to_stack $C_HOSTNAME_TMP $C_HOSTNAME
  ncfm_exit_on_error "ncfm_host_name: ncfm_rollback_add_to_stack failed"
  
  case "$G_NCFM_ACTION" in

    insert)
      ncfm_add_host_name
      ncfm_exit_on_error "ncfm_add_host_name failed"
      ;;
  
    update)
      ncfm_update_host_name
      ncfm_exit_on_error "ncfm_update_host_name failed"
      ;;
     
    delete)
      ncfm_delete_host_name
      ncfm_exit_on_error "ncfm_delete_host_name failed"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  ncfm_rollback_delete_from_stack $C_HOSTNAME_TMP $C_HOSTNAME
  ncfm_exit_on_error "ncfm_host_name: ncfm_rollback_delete_from_stack failed"
  
  return $OK
}
###################
# HOST NAME - END #
###################


#######################
# IPMI BOARD POSITION #
#######################
ncfm_get_ipmi_board_pos()
{
  $X_ECHO "TODO: tega se ni"
  return $E_ERROR
  $X_IPMICLI -p
}

ncfm_ipmi_board_pos()
{
  case "$G_NCFM_ACTION" in

    get)
      ncfm_get_ipmi_board_pos
      ncfm_exit_on_error "ncfm_get_ipmi_board_pos failed"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  return $OK
}
#############################
# IPMI BOARD POSITION - END #
#############################


##############
# INTERFACES #
##############
#this function returns real interface name (for now we only support VLANs and not logical interfaces withoud VLANs)
#first argument is VLAN ID
#secodn argument is interface name
ncfm_get_real_interface_name()
{
  #ncfm_get_real_interface_name needs two argumet
  if [ $# -lt 2 ]
  then
    ncfm_printErr "ncfm_get_real_interface_name: Wrong number of arguments"
    return $E_INTERNAL
  fi
  
  local vlan=$1
  local ifName=$2
  #we check for regular vlan range
  if [ $vlan -lt 1 ] || [ $vlan -gt 4094 ]
  then
    [ $vlan -gt 4094 ] && ncfm_printErr "VLAN ID ($vlan) is out of range. Interface will be set without VLAN!"
    G_INTERFACE_NAME=$ifName
  else
    G_INTERFACE_NAME=$ifName.$vlan
  fi
  
  return $OK
}

#converts dot n9otaion bitmask into a CIDR notation mask
ncfm_dotToCIDR()
{
  #ncfm_dotToCIDR needs one argumet: netmask in dot notation
  if [ $# -lt 1 ]
  then
    ncfm_printErr "ncfm_dotToCIDR: netmask in dot notation is needed"
    return $E_INTERNAL
  fi
  
  IFS=$.
  set -- $1
  unset IFS
  
  local stopLoop=0
  local bitValue=0
  local CIDRmask=0
  local netmask=0
  
  while [ ! -z "$1" ]
  do
    netmask=$1
    shift
    bitValue=128
    while [ "$bitValue" -ge "1" -a "$stopLoop" -ne "1" ]
    do
      if [ "$netmask" -ge "$bitValue" ]
      then
        : $((CIDRmask = $CIDRmask + 1))
      else
        stopLoop=1
        break
      fi
      : $((netmask = $netmask - $bitValue))
      : $((bitValue = $bitValue / 2)) #division by 2
    done #end while
  done #end while
  
  G_TMP=$CIDRmask
  return $OK
}

ncfm_interface_ip()
{
  #ncfm_interface_ip needs one argumet: action
  if [ $# -lt 1 ]
  then
    ncfm_printErr "ncfm_interface_ip: No action"
    return $E_INTERNAL
  fi
  local action=$1
  
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have four argument
  if [ $# -lt 4 ]
  then
    return $E_BAD_ARGS
  fi
  
  local ip
  local netmask
  local vlan
  local ifName
  local ethName
  
  ip=$1
  netmask=$2
  vlan=$3
  ifName=$4
  ncfm_get_real_interface_name $vlan $ifName
  ncfm_exit_on_error "ncfm_get_real_interface_name failed"
  ethName=$G_INTERFACE_NAME
  
  #TODO-this is bash style substitution
  if [ "${netmask:0:1}" != "/" ] 
  then
    ncfm_dotToCIDR $netmask
    netmask="/"$G_TMP;
  fi

  case "$action" in
    attach)
      $X_IP addr add $ip$netmask brd + dev $ethName >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Attaching of IP address failed"
      ;;
    delete)
      $X_IP addr del $ip$netmask dev $ethName >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Deleting of IP address failed"
      ;;
    *)#default
      ncfm_printErr "ncfm_interface_ip: Wrong action"
      exit $E_INTERNAL
      ;;
  esac #end case
  
  return $OK
}

#TODO:we dont support logical interfaces or bonding jet! - support expected for action: create
#for delete i think it already should be supported
ncfm_interface_set_state()
{
  #ncfm_interface_set_state needs one argumet: action
  if [ $# -lt 1 ]
  then
    ncfm_printErr "ncfm_interface_set_state: No action"
    return $E_INTERNAL
  fi
  local action=$1
  
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have two argument
  if [ $# -lt 2 ]
  then
    return $E_BAD_ARGS
  fi
  
  local vlan
  local ifName
  local ethName
  
  vlan=$1
  ifName=$2
  ncfm_get_real_interface_name $vlan $ifName
  ncfm_exit_on_error "ncfm_get_real_interface_name failed"
  ethName=$G_INTERFACE_NAME
  
  case "$action" in
    up)
      $X_IP link set dev $ethName up >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Cannot set interface ($ethName) up"
      ;;
    down)
      $X_IP link set dev $ethName down >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Cannot set interface ($ethName) down"
      ;;
    create)
      $X_VCONFIG add $ifName $vlan >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Cannot create interface ($ethName). $ifName must be up!"
      ;;
    delete)
      $X_VCONFIG rem $ethName >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Cannot delete interface ($ethName)"
      ;;
    *)#default
      ncfm_printErr "ncfm_interface_ip: Wrong action"
      exit $E_INTERNAL
      ;;
  esac #end case
  
  return $OK
}

ncfm_interface()
{
  case "$G_NCFM_ACTION" in

    insert)
      ncfm_interface_set_state create
      ncfm_exit_on_error "ncfm_interface_set_state failed while trying to create vlan"
      ;;
    
    update)
      #TODO
      ncfm_printErr "Not supported yet!"
      return $E_ERROR
      ;;
      
    delete)
      ncfm_interface_set_state delete
      ncfm_exit_on_error "ncfm_interface_set_state failed while trying to delete vlan"
      ;;
      
    up)
      ncfm_interface_set_state up
      ncfm_exit_on_error "ncfm_interface_set_state failed while trying to set state to up"
      ;;
      
    down)
      ncfm_interface_set_state down
      ncfm_exit_on_error "ncfm_interface_set_state failed while trying to set state to down"
      ;;
    
    attach-ip)
      ncfm_interface_ip attach
      ncfm_exit_on_error "ncfm_interface_ip failed while attaching ip"
      ;;
    
    update-ip)
      #TODO
      ncfm_printErr "Not supported yet!"
      return $E_ERROR
      ;;
     
    delete-ip)
      ncfm_interface_ip delete
      ncfm_exit_on_error "ncfm_interface_ip failed while deleting ip"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  return $OK
}
####################
# INTERFACES - END #
####################


#########
# ROUTE #
#########
ncfm_route_ip()
{ 
  #ncfm_route_ip needs one argumet: action
  if [ $# -lt 1 ]
  then
    ncfm_printErr "ncfm_route_ip: No action"
    return $E_INTERNAL
  fi
  local action=$1
  
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have three argument
  if [ $# -lt 3 ]
  then
    return $E_BAD_ARGS
  fi
  
  local dst=$1
  local netmask=$2
  local gw=$3
  
  #TODO-this is bash style substitution
  if [ "${netmask:0:1}" != "/" ] 
  then
    ncfm_dotToCIDR $netmask
    netmask="/"$G_TMP;
  fi
  
  if [ "$1" != "default" ]
  then
    dst=$dst$netmask
  fi
  
  case "$action" in
    insert)
      $X_IP route add $dst via $gw >> $F_DEBUGFILE 2>>$F_ERR_FILE
      ncfm_exit_on_error "Cannot add route $dst via $gw"
      ;;
    delete)
      if [ "$gw" == "0" ]
      then
        $X_IP route del $dst >> $F_DEBUGFILE 2>>$F_ERR_FILE
      else
        $X_IP route del $dst via $gw >> $F_DEBUGFILE 2>>$F_ERR_FILE
      fi
      ncfm_exit_on_error "Cannot delete route $dst via $gw"
      ;;
    *)#default
      ncfm_printErr "ncfm_route_ip: Wrong action"
      exit $E_INTERNAL
      ;;
  esac #end case
  
  return $OK
}

ncfm_route()
{
  case "$G_NCFM_ACTION" in

    insert)
      ncfm_route_ip insert
      ncfm_exit_on_error "ncfm_route_ip failed while trying to insert route"
      ;;
    
    update)
      #TODO
      ncfm_printErr "Not supported yet!"
      return $E_ERROR
      ncfm_route_ip update
      ncfm_exit_on_error "ncfm_route_ip failed while trying to update route"
      ;;
     
    delete)
      ncfm_route_ip delete
      ncfm_exit_on_error "ncfm_route_ip failed while trying to delete route"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  return $OK
}
###############
# ROUTE - END #
###############


#################
# SNMP TRAP TOS #
#################
ncfm_validate_tos()
{
  if [ $1 -lt 256 ]
  then
    return $OK
  else
    ncfm_printErr "TOS or. DSCP is invalid!"
    return $E_BAD_ARGS
  fi
  
}

ncfm_snmp_trap_tos_check_input()
{
  local tos
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have two argument
  if [ $# -lt 2 ]
  then
    return $E_BAD_ARGS
  fi
  
  tos=$2
  
  ncfm_check_if_numeric $tos
  ncfm_exit_on_error "Wrong TOS or DSCP! It should be numeric in decimal notation and lover than 256!"
  
  #if we got dscp, first we need to get tos
  if [ "$1" == "dscp" ]
  then
    : $((tos = $tos * 4)) #shift by 2 on left = multiplay by 4
  fi
  
  ncfm_validate_tos $tos
  ncfm_exit_on_error "Wrong TOS or DSCP! It should be numeric in decimal notation and lover than 256!"
  
  G_TMP_VAR1=$tos
  return $OK
}

ncfm_add_snmp_trap_tos()
{
  $X_SED "/\[$STR_SNMPTRAPTOSCATEGORY\]/a\\$STR_SNMPTRAPTOS = $1" $C_NIESUBAGENT > $C_NIESUBAGENT_TMP.ncfm 2>>$F_ERR_FILE
  ncfm_sed_exit_on_error "ncfm_add_snmp_trap_tos: sed failed"
  
  #content change
  $X_CAT $C_NIESUBAGENT_TMP.ncfm > $C_NIESUBAGENT 2>>$F_ERR_FILE
  ncfm_exit_on_error "ncfm_add_snmp_trap_tos: cat failed"
  
  $X_RM $C_NIESUBAGENT_TMP.ncfm >> $F_DEBUGFILE 2>>$F_ERR_FILE
  
  return $OK
}

ncfm_delete_snmp_trap_tos()
{
  $X_SED -i "/\<$STR_SNMPTRAPTOS\>/ d" $C_NIESUBAGENT >> $F_DEBUGFILE 2>>$F_ERR_FILE
  ncfm_sed_exit_on_error "ncfm_delete_snmp_trap_tos: sed failed"
  return $OK
}

ncfm_update_snmp_trap_tos()
{
  ncfm_delete_snmp_trap_tos $tos
  ncfm_exit_on_error "ncfm_delete_snmp_trap_tos failed"
  ncfm_add_snmp_trap_tos $tos
  ncfm_exit_on_error "ncfm_add_snmp_trap_tos failed"
  return $OK
}

ncfm_snmp_trap_tos()
{
  local tos=0
  ncfm_snmp_trap_tos_check_input
  tos=$G_TMP_VAR1
  
  ncfm_rollback_add_to_stack $C_NIESUBAGENT_TMP $C_NIESUBAGENT
  ncfm_exit_on_error "ncfm_snmp_trap_tos: ncfm_rollback_add_to_stack failed"
  
  case "$G_NCFM_ACTION" in

    update)
      ncfm_update_snmp_trap_tos $tos
      ncfm_exit_on_error "ncfm_update_snmp_trap_tos failed"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  ncfm_restart_service niesubag $X_NIESUBAG restart
  ncfm_exit_on_error "ncfm_snmp_trap_tos: restarting of niesubag failed"
  
  ncfm_rollback_delete_from_stack $C_NIESUBAGENT_TMP $C_NIESUBAGENT
  ncfm_exit_on_error "ncfm_snmp_trap_tos: ncfm_rollback_delete_from_stack failed"
  
  return $OK
}

#######################
# SNMP TRAP TOS - END #
#######################


#############
# TIME ZONE #
#############
ncfm_offset2timezone()
{
  #TODO
  ncfm_printErr "Not supported yet!"
  return $E_ERROR
  return $OK
}

ncfm_validate_timezone()
{
  local timezone=$1
  
  #TODO: delete above line
  echo $timezone
  
  #we check if timezone file exists
  if [ ! -s $D_ZONEINFO$timezone ]
  then
    ncfm_printErr "Time zone is invalid"
    return $E_BAD_ARGS
  fi
  
  return $OK
}

ncfm_update_time_zone()
{
  local timezone
  set -- $G_NCFM_NEW_OPTIONS
  
  #check if we have two argument
  if [ $# -lt 2 ]
  then
    return $E_BAD_ARGS
  fi
  
  timezone=$2
  
  #if we got offset, first we need to get timezone
  [ "$1" == "offset" ] && ncfm_offset2timezone
  
  ncfm_validate_timezone $timezone
  ncfm_exit_on_error "Wrong timezone"
  
  $X_RM $C_LOCALTIME >> $F_DEBUGFILE 2>>$F_ERR_FILE
  $X_LN -sf $D_ZONEINFO$timezone $C_LOCALTIME >> $F_DEBUGFILE 2>>$F_ERR_FILE
  
  return $OK
}

ncfm_time_zone()
{
  ncfm_rollback_add_to_stack $C_NTP_TMP $C_NTP
  ncfm_exit_on_error "ncfm_time_zone: ncfm_rollback_add_to_stack failed"
  
  case "$G_NCFM_ACTION" in

    update)
      ncfm_update_time_zone
      ncfm_exit_on_error "ncfm_update_time_zone failed"
      ;;
  
    *)#default
      ncfm_printErr "Wrong action"
      exit $E_BAD_ARGS
      ;;

  esac #end case
  
  ncfm_rollback_delete_from_stack $C_NTP_TMP $C_NTP
  ncfm_exit_on_error "ncfm_time_zone: ncfm_rollback_delete_from_stack failed"
  
  return $OK
}
###################
# TIME ZONE - END #
###################
